require 'helper'

module HasVersions
  module Version
    class DiffTest < MiniTest::Unit::TestCase
      class DiffRecord < TestRecord
        attr_accessor :name
        attr_accessor :color
        attr_accessor :weight

        versioning_configuration.attribute :name
        versioning_configuration.attribute :weight

        def self.versioning_decode_value(name, value)
          "decoded: #{value}"
        end
      end

      def test_diff_with_parent
        target = DiffRecord.new
        parent = DiffRecord.version_model.new(target: target, snapshot: {name: 'foo', color: 'green', weight: '4.3'})
        child = DiffRecord.version_model.new(target: target, parent: parent, snapshot: {name: 'bar', color: 'orange', weight: '4.3'})

        diff = child.diff

        assert_equal({'name' => ['decoded: foo', 'decoded: bar']}, diff)
      end

      def test_diff_with_parameter
        target = DiffRecord.new
        this_version = DiffRecord.version_model.new(target: target, snapshot: {name: 'foo', color: 'green', weight: '4.3'})
        other_version = DiffRecord.version_model.new(target: target, snapshot: {name: 'bar', color: 'orange', weight: '4.3'})

        diff = this_version.diff(other_version)

        assert_equal({'name' => ['decoded: bar', 'decoded: foo']}, diff)
      end

      def test_diff_with_nil
        target = DiffRecord.new
        version = DiffRecord.version_model.new(target: target, snapshot: {name: 'bar', color: 'orange', weight: '4.3'})

        diff = version.diff

        assert_equal({'name' => [nil, 'decoded: bar'], 'weight' => [nil, 'decoded: 4.3']}, diff)
      end

      def test_modifications
        target = DiffRecord.new
        parent = DiffRecord.version_model.new(target: target, snapshot: {'color' => 'green', 'name' => 'joe', 'weight' => '50'})
        version = DiffRecord.version_model.new(target: target, parent: parent, snapshot: {'color' => 'green', 'name' => 'sam', 'weight' => '129'})
        other = DiffRecord.version_model.new(target: target, snapshot: {'color' => 'blue', 'name' => 'sam', 'weight' => '100'})

        assert_equal ['name'], version.applied_modifications(other)
        assert_equal ['weight'], version.unused_modifications(other)
      end

      def test_decoded_snapshot
        target = DiffRecord.new
        version = DiffRecord.version_model.new(target: target, snapshot: {name: 'foo', color: 'green', weight: '4.3'})

        assert_equal({"name" => "decoded: foo", "weight" => "decoded: 4.3"}, version.decoded_snapshot)
      end
    end
  end
end